import datetime
import cnlunar
import pymysql
from commondata import *


class ShengChen:
    def __init__(self, date=datetime.datetime(1901, 1, 1, 0, 0, 0, 0)):
        self.date = date
        self.lunarData = cnlunar.Lunar(self.date, godType='8char')

    def hour_to_shichen(self):
        if self.date.hour not in range(0, 24):
            print('out of range')
            return False
        elif self.date.hour in [0, 23]:
            shichen = 0
        elif self.date.hour in [1, 2]:
            shichen = 1
        elif self.date.hour % 2 == 0:
            shichen = self.date.hour // 2
        else:
            shichen = (self.date.hour + 1) // 2
        return shichen

    def get_year_weight(self):
        for items in weight_of_year:
            if items["name"] == self.lunarData.year8Char:
                return items["weight"]

    def get_month_weight(self):
        return weight_of_month[self.lunarData.lunarMonth - 1]

    def get_day_weight(self):
        return weight_of_day[self.lunarData.lunarDay - 1]

    def get_hour_weight(self):
        return weight_of_hour[self.hour_to_shichen()]

    def get_total_weight(self):
        return self.get_hour_weight() + self.get_day_weight() + self.get_month_weight() + self.get_year_weight()

    def insert_year_weight_into_db(self):
        try:
            cursor.execute(
                "insert into year_weight(year, year8char, weight) values(%s, %s, %s)",
                (self.date, self.lunarData.year8Char, self.get_year_weight())
            )
            db.commit()
            print(f'insert success for date {self.date}')
        except:
            pass

    def insert_into_db(self):
        try:
            cursor.execute(
                "insert into shengchen(datetime,year8Char,lunarMonth,lunarDay,shichen,weight) values(%s, %s, %s, %s, %s, %s)",
                (self.date, self.lunarData.year8Char, self.lunarData.lunarMonth, self.lunarData.lunarDay,
                 self.hour_to_shichen(), self.get_total_weight())
            )
            db.commit()
            print(f'insert success for date {self.date}')
        except:
            pass


def calculate_180_years():
    start_date = datetime.datetime(1901, 1, 1, 0, 0, 1, 0)
    end_date = datetime.datetime(2081, 12, 31, 23, 0, 1, 0)

    while start_date <= end_date:
        shengchen1 = ShengChen(start_date)
        # print(shengchen1.lunarData.year8Char)
        # print(shengchen1.lunarData.lunarMonth)
        # print(shengchen1.lunarData.lunarDay)
        # print(shengchen1.get_total_weight())
        shengchen1.insert_into_db()
        start_date = start_date + datetime.timedelta(hours = 2)
    print('finish!')


def insert_year_weight():
    start_year = 1901
    end_year = 2081
    while start_year <= end_year:
        shengchen1 = ShengChen(datetime.datetime(start_year, 6, 1, 0, 0, 1, 0))
        shengchen1.insert_year_weight_into_db()
        start_year = start_year + 1
    print('finish')


if __name__ == '__main__':
    db = pymysql.connect(host='localhost', port=3306,
                         user='root',
                         password='password',
                         database='yuantiangang')

    cursor = db.cursor()

    calculate_180_years()
    insert_year_weight()
    db.close()